VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsPopupMenu"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' Autor: F. Schler (frank@activevb.de)
' Datum: 03/2023

Option Explicit

' ----==== Const ====----
Private Const WindowsUIPopupsPopupMenu As String = "Windows.UI.Popups.PopupMenu"
Private Const IID_IPopupMenu As String = "{4e9bc6dc-880d-47fc-a0a1-72b639e62559}"

' ----==== Enums ====----
Private Enum vtb_Interfaces

    ' IPopupMenu
    IPopupMenu_GetCommands = 6
    IPopupMenu_ShowAsync = 7
    IPopupMenu_ShowAsyncWithRect = 8
    IPopupMenu_ShowAsyncWithRectAndPlacement = 9
    
End Enum

' ----==== Variablen ====----
Private m_pIPopupMenu As Long

' ----==== Class ====----
Private Sub Class_Initialize()
    If GetActivateInstance(WindowsUIPopupsPopupMenu, _
                           IID_IPopupMenu, _
                           m_pIPopupMenu) Then
    End If
End Sub

Private Sub Class_Terminate()
    Call ReleaseIfc(m_pIPopupMenu)
End Sub

' ----==== Properties ====----
Public Property Get Ifc() As Long
    Ifc = m_pIPopupMenu
End Property

Public Property Let Ifc(ByVal pIPopupMenu As Long)
    Call ReleaseIfc(m_pIPopupMenu)
    m_pIPopupMenu = pIPopupMenu
End Property

' IPopupMenu
Public Property Get Commands() As clsUICommandList
    Dim Ret As New clsUICommandList
    If m_pIPopupMenu <> 0& Then
        Dim pIUICommandList As Long
        If Invoke(m_pIPopupMenu, _
                  IPopupMenu_GetCommands, _
                  VarPtr(pIUICommandList)) = S_OK Then
            Ret.Ifc = pIUICommandList
        End If
    End If
    Set Commands = Ret
End Property

' ----==== Functions ====----
' IPopupMenu
Friend Function ShowAsync(ByVal ownerHwnd As Long, _
                          ByRef invocationPoint As Windows_Foundation_Point) As clsUICommand
    Dim Ret As New clsUICommand
    If m_pIPopupMenu <> 0& Then
        If InitWithWindow(m_pIPopupMenu, ownerHwnd) Then
            Dim pIUICommand As Long
            If Invoke(m_pIPopupMenu, _
                      IPopupMenu_ShowAsync, _
                      invocationPoint.X, _
                      invocationPoint.Y, _
                      VarPtr(pIUICommand)) = S_OK Then
                If Await(pIUICommand) Then
                    Ret.Ifc = pIUICommand
                End If
            End If
        End If
    End If
    Set ShowAsync = Ret
End Function

Friend Function ShowAsyncWithRect(ByVal ownerHwnd As Long, _
                                  ByRef selection As Windows_Foundation_Rect) As clsUICommand
    Dim Ret As New clsUICommand
    If m_pIPopupMenu <> 0& Then
        If InitWithWindow(m_pIPopupMenu, ownerHwnd) Then
            Dim pIUICommand As Long
            If Invoke(m_pIPopupMenu, _
                      IPopupMenu_ShowAsyncWithRect, _
                      selection.X, _
                      selection.Y, _
                      selection.Width, _
                      selection.Height, _
                      VarPtr(pIUICommand)) = S_OK Then
                If Await(pIUICommand) Then
                    Ret.Ifc = pIUICommand
                End If
            End If
        End If
    End If
    Set ShowAsyncWithRect = Ret
End Function

Friend Function ShowAsyncWithRectAndPlacement(ByVal ownerHwnd As Long, _
                                              ByRef selection As Windows_Foundation_Rect, _
                                              ByVal preferredPlacement As Windows_UI_Popups_Placement) As clsUICommand
    Dim Ret As New clsUICommand
    If m_pIPopupMenu <> 0& Then
        If InitWithWindow(m_pIPopupMenu, ownerHwnd) Then
            Dim pIUICommand As Long
            If Invoke(m_pIPopupMenu, _
                      IPopupMenu_ShowAsyncWithRectAndPlacement, _
                      selection.X, _
                      selection.Y, _
                      selection.Width, _
                      selection.Height, _
                      preferredPlacement, _
                      VarPtr(pIUICommand)) = S_OK Then
                If Await(pIUICommand) Then
                    Ret.Ifc = pIUICommand
                End If
            End If
        End If
    End If
    Set ShowAsyncWithRectAndPlacement = Ret
End Function

' ----==== Invoke Interface ====----
Private Function Invoke(ByVal pInterface As Long, _
                        ByVal vtb As vtb_Interfaces, _
                        ParamArray var()) As Variant
    If pInterface <> 0& Then
        Invoke = OleInvoke(pInterface, vtb, var)
    End If
End Function



