VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsWindowFlyout"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' Autor: F. Schler (frank@activevb.de)
' Datum: 03/2023

Option Explicit

' ----==== Const ====----
Private Const WindowsUICoreCoreWindowFlyout As String = "Windows.UI.Core.CoreWindowFlyout"
Private Const IID_ICoreWindowFlyout As String = "{e89d854d-2050-40bb-b344-f6f355eeb314}"
Private Const IID_ICoreWindowFlyoutFactory As String = "{dec4c6c4-93e8-4f7c-be27-cefaa1af68a7}"

' ----==== Enums ====----
Private Enum vtb_Interfaces

    ' ICoreWindowFlyoutFactory
    ICoreWindowFlyoutFactory_Create = 6
    ICoreWindowFlyoutFactory_CreateWithTitle = 7
    
    ' ICoreWindowFlyout
    ICoreWindowFlyout_AddShowing = 6
    ICoreWindowFlyout_RemoveShowing = 7
    ICoreWindowFlyout_GetMaxSize = 8
    ICoreWindowFlyout_GetMinSize = 9
    ICoreWindowFlyout_GetTitle = 10
    ICoreWindowFlyout_PutTitle = 11
    ICoreWindowFlyout_GetIsInteractionDelayed = 12
    ICoreWindowFlyout_PutIsInteractionDelayed = 13
    ICoreWindowFlyout_GetCommands = 14
    ICoreWindowFlyout_GetDefaultCommandIndex = 15
    ICoreWindowFlyout_PutDefaultCommandIndex = 16
'    ICoreWindowFlyout_GetBackButtonCommand = 17
    ICoreWindowFlyout_PutBackButtonCommand = 18
    ICoreWindowFlyout_ShowAsync = 19

End Enum

' ----==== Variablen ====----
Private m_pICoreWindowFlyout As Long
Private m_pICoreWindowFlyoutFactory As Long
Private m_pCoreWindowPopupShowingEvent As Long
Private m_pCoreWindowPopupShowingEventCookie As Long

' ----==== Events ====----
Public Event CoreWindowPopupShowing(ByVal sender As Long, ByVal args As clsCoreWindowPopupShowingEventArgs)

' ----==== Class ====----
Private Sub Class_Initialize()
    If GetActivationFactory(WindowsUICoreCoreWindowFlyout, _
                            IID_ICoreWindowFlyoutFactory, _
                            m_pICoreWindowFlyoutFactory) Then
    End If
End Sub

Private Sub Class_Terminate()
    If m_pCoreWindowPopupShowingEventCookie <> 0& Then
        If Invoke(m_pICoreWindowFlyout, _
                  ICoreWindowFlyout_RemoveShowing, _
                  m_pCoreWindowPopupShowingEvent, _
                  m_pCoreWindowPopupShowingEventCookie) = S_OK Then
            m_pCoreWindowPopupShowingEventCookie = 0&
        End If
    End If
    Call ReleaseIfc(m_pICoreWindowFlyout)
    Call ReleaseIfc(m_pICoreWindowFlyoutFactory)
End Sub

' ----==== Properties ====----
Public Property Get Ifc() As Long
    Ifc = m_pICoreWindowFlyout
End Property

Public Property Let Ifc(ByVal pICoreWindowFlyout As Long)
    If m_pCoreWindowPopupShowingEventCookie <> 0& Then
        If Invoke(m_pICoreWindowFlyout, _
                  ICoreWindowFlyout_RemoveShowing, _
                  m_pCoreWindowPopupShowingEvent, _
                  m_pCoreWindowPopupShowingEventCookie) = S_OK Then
            m_pCoreWindowPopupShowingEventCookie = 0&
        End If
    End If
    Call ReleaseIfc(m_pICoreWindowFlyout)
    m_pICoreWindowFlyout = pICoreWindowFlyout
    m_pCoreWindowPopupShowingEvent = modCoreWindowPopupShowingEvent.Create(Me)
    If Invoke(m_pICoreWindowFlyout, _
              ICoreWindowFlyout_AddShowing, _
              m_pCoreWindowPopupShowingEvent, _
              VarPtr(m_pCoreWindowPopupShowingEventCookie)) = S_OK Then
    End If
End Property

' ICoreWindowFlyout
Friend Property Get MaxSize() As Windows_Foundation_Size
    Dim Ret As Windows_Foundation_Size
    If m_pICoreWindowFlyout <> 0& Then
        Dim value As Windows_Foundation_Size
        If Invoke(m_pICoreWindowFlyout, _
                  ICoreWindowFlyout_GetMaxSize, _
                  VarPtr(value)) = S_OK Then
            Ret = value
        End If
    End If
    MaxSize = Ret
End Property

Friend Property Get MinSize() As Windows_Foundation_Size
    Dim Ret As Windows_Foundation_Size
    If m_pICoreWindowFlyout <> 0& Then
        Dim value As Windows_Foundation_Size
        If Invoke(m_pICoreWindowFlyout, _
                  ICoreWindowFlyout_GetMinSize, _
                  VarPtr(value)) = S_OK Then
            Ret = value
        End If
    End If
    MinSize = Ret
End Property

Public Property Get Title() As String
    Dim Ret As String
    If m_pICoreWindowFlyout <> 0& Then
        Dim hString As Long
        If Invoke(m_pICoreWindowFlyout, _
                  ICoreWindowFlyout_GetTitle, _
                  VarPtr(hString)) = S_OK Then
            Ret = GetWindowsString(hString)
        End If
    End If
    Title = Ret
End Property

Public Property Let Title(ByVal value As String)
    If m_pICoreWindowFlyout <> 0& Then
        Dim hString As Long
        hString = CreateWindowsString(value)
        If Invoke(m_pICoreWindowFlyout, _
                  ICoreWindowFlyout_PutTitle, _
                  hString) = S_OK Then
        End If
        Call DeleteWindowsString(hString)
    End If
End Property

Public Property Get IsInteractionDelayed() As Boolean
    Dim Ret As Boolean
    If m_pICoreWindowFlyout <> 0& Then
        Dim value As Long
        If Invoke(m_pICoreWindowFlyout, _
                  ICoreWindowFlyout_GetIsInteractionDelayed, _
                  VarPtr(value)) = S_OK Then
            Ret = CBool(value)
        End If
    End If
    IsInteractionDelayed = Ret
End Property

Public Property Let IsInteractionDelayed(ByVal value As Boolean)
    If m_pICoreWindowFlyout <> 0& Then
        If Invoke(m_pICoreWindowFlyout, _
                  ICoreWindowFlyout_PutIsInteractionDelayed, _
                  value) = S_OK Then
        End If
    End If
End Property

Public Property Get Commands() As clsUICommandList
    Dim Ret As New clsUICommandList
    If m_pICoreWindowFlyout <> 0& Then
        Dim pIUICommandList As Long
        If Invoke(m_pICoreWindowFlyout, _
                  ICoreWindowFlyout_GetCommands, _
                  VarPtr(pIUICommandList)) = S_OK Then
            Ret.Ifc = pIUICommandList
        End If
    End If
    Set Commands = Ret
End Property

Public Property Get DefaultCommandIndex() As Long
    Dim Ret As Long
    If m_pICoreWindowFlyout <> 0& Then
        Dim value As Long
        If Invoke(m_pICoreWindowFlyout, _
                  ICoreWindowFlyout_GetDefaultCommandIndex, _
                  VarPtr(value)) = S_OK Then
            Ret = value
        End If
    End If
    DefaultCommandIndex = Ret
End Property

Public Property Let DefaultCommandIndex(ByVal value As Long)
    If m_pICoreWindowFlyout <> 0& Then
        If Invoke(m_pICoreWindowFlyout, _
                  ICoreWindowFlyout_PutDefaultCommandIndex, _
                  value) = S_OK Then
        End If
    End If
End Property

'    ICoreWindowFlyout_GetBackButtonCommand = 17

Public Property Let BackButtonCommand(ByVal value As clsUICommandInvokedHandler)
    If m_pICoreWindowFlyout <> 0& And _
       value.Ifc <> 0& Then
        If Invoke(m_pICoreWindowFlyout, _
                  ICoreWindowFlyout_PutBackButtonCommand, _
                  value.Ifc) = S_OK Then
        End If
    End If
End Property

' ----==== Functions ====----
' ICoreWindowFlyoutFactory
Friend Function Create(ByRef position As Windows_Foundation_Point) As Boolean
    Dim Ret As Boolean
    If m_pICoreWindowFlyoutFactory <> 0& Then
        Dim pICoreWindowFlyout As Long
        If Invoke(m_pICoreWindowFlyoutFactory, _
                  ICoreWindowFlyoutFactory_Create, _
                  position.X, _
                  position.Y, _
                  VarPtr(pICoreWindowFlyout)) = S_OK Then
            Ret = True
            Me.Ifc = pICoreWindowFlyout
        End If
    End If
    Create = Ret
End Function

Friend Function CreateWithTitle(ByRef position As Windows_Foundation_Point, _
                                ByVal Title As String) As Boolean
    Dim Ret As Boolean
    If m_pICoreWindowFlyoutFactory <> 0& Then
        Dim hTitle As Long
        hTitle = CreateWindowsString(Title)
        Dim pICoreWindowFlyout As Long
        If Invoke(m_pICoreWindowFlyoutFactory, _
                  ICoreWindowFlyoutFactory_CreateWithTitle, _
                  position.X, _
                  position.Y, _
                  hTitle, _
                  VarPtr(pICoreWindowFlyout)) = S_OK Then
            Ret = True
            Me.Ifc = pICoreWindowFlyout
        End If
        Call DeleteWindowsString(hTitle)
    End If
    CreateWithTitle = Ret
End Function

' ICoreWindowFlyout
Public Function ShowAsync(ByVal ownerHwnd As Long) As clsUICommand
    Dim Ret As New clsUICommand
    If m_pICoreWindowFlyout <> 0& Then
        If InitWithWindow(m_pICoreWindowFlyout, ownerHwnd) Then
            Dim pIUICommand As Long
            If Invoke(m_pICoreWindowFlyout, _
                      ICoreWindowFlyout_ShowAsync, _
                      VarPtr(pIUICommand)) = S_OK Then
                If Await(pIUICommand) Then
                    Ret.Ifc = pIUICommand
                End If
            End If
        End If
    End If
    Set ShowAsync = Ret
End Function

' ----==== Invoke Handler ====----
Public Sub CoreWindowPopupShowingEvent(ByVal sender As Long, ByVal args As Long)
    Dim CoreWindowPopupShowingEventArgs As New clsCoreWindowPopupShowingEventArgs
    CoreWindowPopupShowingEventArgs.Ifc = args
    RaiseEvent CoreWindowPopupShowing(sender, CoreWindowPopupShowingEventArgs)
End Sub

' ----==== Invoke Interface ====----
Private Function Invoke(ByVal pInterface As Long, _
                        ByVal vtb As vtb_Interfaces, _
                        ParamArray var()) As Variant
    If pInterface <> 0& Then
        Invoke = OleInvoke(pInterface, vtb, var)
    End If
End Function


